import processing.opengl.*;
import SimpleOpenNI.*;
import javax.sound.midi.*;

import saito.objloader.*;
SimpleOpenNI  kinect;

SkeletonRecorder recorder;
boolean recording = false;
float offByDistance = 0.0;

OBJModel model;
float rotX = 0;
float rotY = -1.6;
float changeSize = 20;


void setup() {
  size(1028, 768, OPENGL);


  //model = new OBJModel(this, "Citizen.obj", "relative", TRIANGLES);
  model = new OBJModel(this, dataPath("Citizen.obj"), "relative", TRIANGLES);
  model.translateToCenter();
  kinect = new SimpleOpenNI(this);
  kinect.enableDepth();
  kinect.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL);
  kinect.setMirror(true);
  // initialize our recorder and
  // tell it to track left hand
  recorder = new SkeletonRecorder(kinect, SimpleOpenNI.SKEL_RIGHT_ELBOW);
  // load a font
  PFont font = createFont("Verdana", 40);
  textFont(font);
}

void draw() {
  background(0);
  kinect.update();
  // these are to make our spheres look nice
  lights();
  noStroke();
  // create heads-up display
  fill(255);
  text("totalFrames: " + recorder.frames.size(), 5, 50);
  text("recording: " + recording, 5, 100);
  text("currentFrame: " + recorder.currentFrame, 5, 150 );
  translate(width/2, height/2, 0);
  rotateX(radians(180));

  IntVector userList = new IntVector();
  kinect.getUsers(userList);
  if (userList.size() > 0) {
    int userId = userList.get(0);
    recorder.setUser(userId);
    if ( kinect.isTrackingSkeleton(userId)) {
      if (recording) {
        PVector currentPosition = new PVector();
        kinect.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_ELBOW, currentPosition);
        PMatrix3D orientation = new PMatrix3D();
        kinect.getJointOrientationSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_ELBOW, orientation);
        draw3D(currentPosition, orientation);
        recorder.recordFrame();
      }
      else {
        // if we're playing access the recorded joint position
        PVector recordedPosition = recorder.getPosition();
        PMatrix3D  recordedOrientation = recorder.getOrientation();
        draw3D(recordedPosition, recordedOrientation);
        recorder.nextFrame();
      }
    }
  }
}


void startMidi() {
  try {
    File midiFile = new File(dataPath("love.midi"));
    Sequencer sequencer = MidiSystem.getSequencer();
    sequencer.open();
    Sequence sequence = MidiSystem.getSequence(midiFile);
    sequencer.setSequence(sequence);
    sequencer.start();
  } 
  catch(Exception e) {
  }
}

void draw3D(PVector _p, PMatrix3D _o) {
  lights();
  pushMatrix();
  fill(255, 0, 0);
  translate(_p.x, _p.y, _p.z);  
  _o.rotateY(rotY);
  applyMatrix(_o);
  scale(changeSize);
  noStroke();
  model.draw();
  popMatrix();
}

void keyPressed() {
   sequencer.stop();
 sequencer.start();
  recording = !recording;
}

// user-tracking callbacks!
void onNewUser(int userId) {
  println("start pose detection");
  kinect.startPoseDetection("Psi", userId);
}

void onEndCalibration(int userId, boolean successful) {
  if (successful) { 
    println("  User calibrated !!!");
    kinect.startTrackingSkeleton(userId);
    recording = true;
    startMidi();
  } 
  else { 
    println("  Failed to calibrate user !!!");
    kinect.startPoseDetection("Psi", userId);
  }
}

void onStartPose(String pose, int userId) {
  println("Started pose for user");
  kinect.stopPoseDetection(userId); 
  kinect.requestCalibrationSkeleton(userId, true);
}

